@@ -1,8 +1,11 @@
 #include <QtGui>
 #include <QtDebug>
 #include <QInputDialog>
-
+#ifdef Q_WS_MAEMO_5
+#include <QtMaemo5/QMaemo5InformationBox>
+#endif
 #include "mainwindow.h"
+#include "optiondialog.h"
 #include "time.h"
 
 //#define AVOID_INPUT_DIALOG 0
@@ -22,16 +25,34 @@ MainWindow::MainWindow()
     connect(mediaObject, SIGNAL(aboutToFinish()), this, SLOT(aboutToFinish()));
     connect (&plman, SIGNAL (playlistChanged (int)), this, SLOT (playlistChanged(int)));
     connect (&plman, SIGNAL (itemUpdated(int)), this, SLOT (itemUpdated (int)));
+    connect (&plman, SIGNAL (itemRemoved(int)), this, SLOT (itemRemoved (int)));
 
     Phonon::createPath(mediaObject, audioOutput);
 
     qsrand (time (NULL));
     repeat = settings.value("repeat", false).toBool();
     shuffle = settings.value("shuffle", false).toBool();
+    QStringList allowedHeaders;
+    allowedHeaders << "Artist" << "Title" << "Album" << "Controls";
+    foreach (QString h, settings.value ("headers", QStringList()).toStringList())
+    {
+        if (allowedHeaders.indexOf(h) >= 0)
+            headers << h;
+    }
+    if (!headers.size())
+        headers << "Artist" << "Title" << "Album";
+    
     setupShuffleList();
     setupActions();
     setupMenus();
-    setupUi();
+/*    foreach (QString s, Phonon::BackendCapabilities::availableMimeTypes())
+        qDebug () << s;*/
+    
+/*    if (settings.value("uiflipped", false).toBool())
+        setupUiFlipped();
+    else*/
+    setupUi ();
+    show ();
     timeLcd->display("00:00:00");
     plman.addStringList(settings.value("lastPlaylist").toStringList());
     setupShuffleList();
@@ -40,6 +61,7 @@ MainWindow::MainWindow()
         setItem (curind, false);
     audioOutput->setVolume(settings.value("volume", .5).toReal());
     QApplication::setWindowIcon(QIcon (QPixmap (":images/tomamp")));
+    setOrientation();
 }
 
 MainWindow::~MainWindow()
@@ -49,6 +71,7 @@ MainWindow::~MainWindow()
     settings.setValue("lastPlaylist", plman.playlistStrings());
     settings.setValue("volume", audioOutput->volume());
     settings.setValue("currentIndex", plman.indexOf(mediaObject->currentSource()));
+    settings.setValue("headers", headers);
     for (int i = 0; i < musicTable->columnCount(); ++i)
     {
         QString lab = QString ("colWidth_%1").arg (i);
@@ -56,13 +79,34 @@ MainWindow::~MainWindow()
     }
 }
 
+void MainWindow::setOrientation ()
+{
+#ifdef Q_WS_MAEMO_5
+    QString orient = settings.value("orientation", "Automatic").toString();
+    if (orient == "Portrait")
+    {
+        setAttribute(Qt::WA_Maemo5PortraitOrientation, true);
+    }
+    else if (orient == "Landscape")
+    {
+        setAttribute(Qt::WA_Maemo5LandscapeOrientation, true);
+    }
+    else
+    {
+        setAttribute(Qt::WA_Maemo5AutoOrientation, true);
+    }
+#endif
+}
+
 void MainWindow::addFiles()
 {
     QString folder = settings.value("LastFolder").toString();
     if (folder.isEmpty())
         folder = QDesktopServices::storageLocation(QDesktopServices::MusicLocation);
+    QString ext = "*." + plman.allowedExt().join(" *.");
+    ext = "Music files (" + ext + ");;Playlists (*.m3u *.pls)";
     QStringList files = QFileDialog::getOpenFileNames(this, tr("Select Files To Add"),
-                    folder, "Music files (*.mp3 *.ogg *.wav *.flac);;Playlists (*.m3u *.pls)");
+                                                      folder, ext, 0, QFileDialog::DontUseNativeDialog);
 
     if (files.isEmpty())
         return;
@@ -122,11 +166,12 @@ void MainWindow::addUrl()
 
 void MainWindow::about()
 {
-    QMessageBox::information(this, tr("About TomAmp v0.1"),
+    QMessageBox::information(this, tr("About TomAmp v0.2"),
         tr("TomAmp is a simple playlist-based music player.\n\n"
         "(c) 2010 Tamas Marki <tmarki@gmail.com>\n\n"
         "Please send comments and bug reports to the above e-mail address.\n\n"
-        "Icons by http://itweek.deviantart.com/"));
+        "Icons by http://itweek.deviantart.com/\n\n"
+        "Special thanks to Attila Csipa"));
 }
 
 void MainWindow::stateChanged(Phonon::State newState, Phonon::State /* oldState */)
@@ -147,13 +192,16 @@ void MainWindow::stateChanged(Phonon::State newState, Phonon::State /* oldState
             next ();
             break;
         case Phonon::PlayingState:
-            setWindowTitle(mediaObject->metaData().value("TITLE") + " - TomAmp");
+            setWindowTitle(mediaObject->metaData().value("TITLE") + "(" + mediaObject->metaData().value("ARTIST") + ") - TomAmp");
             pauseAction->setVisible(true);
             playAction->setVisible (false);
             playAction->setEnabled(false);
             pauseAction->setEnabled(true);
             stopAction->setEnabled(true);
-            //lastPlayed = plman.indexOf(mediaObject->currentSource());
+            unhighlightRow(lastPlayed);
+            lastPlayed = plman.indexOf(mediaObject->currentSource());
+            highlightRow(plman.indexOf(mediaObject->currentSource()));
+            musicTable->selectRow(plman.indexOf(mediaObject->currentSource()));
             break;
         case Phonon::StoppedState:
             setWindowTitle("TomAmp");
@@ -184,6 +232,12 @@ void MainWindow::next()
     bool wasPlaying = isPlaying;
     if (mediaObject->state () == Phonon::ErrorState)
         wasPlaying = true;
+    if (mediaObject->queue().size())
+    {
+        setItem (plman.indexOf(mediaObject->queue()[0]), wasPlaying);
+        mediaObject->queue().clear();
+        return;
+    }
     int index = plman.indexOf(mediaObject->currentSource());
     if (shuffle)
     {
@@ -233,8 +287,8 @@ void MainWindow::setItem(int i, bool doplay)
 {
     if (i < plman.size() && i >= 0)
     {
-        if (lastPlayed >= 0)
-            unhighlightRow(lastPlayed);
+/*        if (lastPlayed >= 0)
+            unhighlightRow(lastPlayed);*/
         if (shuffle)
         {
             mediaObject->setCurrentSource(plman.at (shuffleList[i]));
@@ -396,6 +450,8 @@ void MainWindow::sourceChanged(const Phonon::MediaSource &source)
 
 void MainWindow::aboutToFinish()
 {
+    if (mediaObject->queue().size())
+        return;
     int index = plman.indexOf(mediaObject->currentSource()) + 1;
     if (shuffle)
     {
@@ -441,6 +497,9 @@ void MainWindow::setupActions()
     stopAction->setDisabled(true);
     nextAction = new QAction(QIcon (QPixmap (":images/next")), "", this);
     nextAction->setShortcut(tr("Ctrl+N"));
+    upAction = new QAction (QString::fromUtf8("▲"), this);
+    downAction = new QAction (QString::fromUtf8("▼"), this);
+    delAction = new QAction (QString::fromUtf8("╳"), this);
     previousAction = new QAction(QIcon (QPixmap (":images/previous")), "", this);
     previousAction->setShortcut(tr("Ctrl+R"));
     if (repeat)
@@ -457,6 +516,7 @@ void MainWindow::setupActions()
     shuffleAction->setCheckable(true);
     shuffleAction->setChecked(shuffle);
     shuffleAction->setShortcut(tr("Ctrl+H"));
+    enqueueActionButton = new QAction (tr ("E"), this);
     volumeAction = new QAction(QIcon (QPixmap (":images/volume")), "", this);
     volumeAction->setCheckable(true);
     volumeAction->setShortcut(tr("Ctrl+V"));
@@ -472,6 +532,8 @@ void MainWindow::setupActions()
     loadPlaylistAction->setShortcut(tr("Ctrl+L"));
     clearPlaylistAction = new QAction (tr("&Clear Playlist"), this);
     clearPlaylistAction->setShortcut(tr("Ctrl+C"));
+    optionAction = new QAction (tr("Op&tions"), this);
+    optionAction->setShortcut(tr("Ctrl+T"));
     exitAction = new QAction(tr("E&xit"), this);
     exitAction->setShortcut(tr("Ctrl+X"));
     aboutAction = new QAction(tr("A&bout"), this);
@@ -487,6 +549,7 @@ void MainWindow::setupActions()
     connect(repeatAction, SIGNAL(triggered()), this, SLOT(repeatToggle()));
     connect(shuffleAction, SIGNAL(triggered()), this, SLOT(shuffleToggle()));
     connect(volumeAction, SIGNAL(triggered()), this, SLOT(volumeToggle()));
+    connect(enqueueActionButton, SIGNAL(triggered()), this, SLOT(enqueueSelected()));
 
     connect(addFilesAction, SIGNAL(triggered()), this, SLOT(addFiles()));
     connect(addFoldersAction, SIGNAL(triggered()), this, SLOT(addFolder()));
@@ -494,8 +557,12 @@ void MainWindow::setupActions()
     connect (savePlaylistAction, SIGNAL (triggered()), this, SLOT (savePlaylist()));
     connect (loadPlaylistAction, SIGNAL (triggered()), this, SLOT (loadPlaylist()));
     connect (clearPlaylistAction, SIGNAL (triggered()), &plman, SLOT (clearPlaylist()));
+    connect (optionAction, SIGNAL (triggered()), this, SLOT (showOptions()));
     connect (nextAction, SIGNAL(triggered()), this, SLOT(next()));
     connect (previousAction, SIGNAL(triggered()), this, SLOT(previous()));
+    connect (upAction, SIGNAL(triggered()), this, SLOT(upSelected()));
+    connect (downAction, SIGNAL(triggered()), this, SLOT(downSelected()));
+    connect (delAction, SIGNAL(triggered()), this, SLOT(removeSelectedItem()));
     connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));
     connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));
     connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
@@ -554,7 +621,7 @@ void MainWindow::volumeToggle ()
 void MainWindow::play()
 {
     mediaObject->play();
-    lastPlayed = plman.indexOf(mediaObject->currentSource());
+//    lastPlayed = plman.indexOf(mediaObject->currentSource());
     highlightRow(lastPlayed);
     isPlaying = true;
 }
@@ -576,6 +643,7 @@ void MainWindow::setupMenus()
     fileMenu->addAction(savePlaylistAction);
     fileMenu->addAction(loadPlaylistAction);
     fileMenu->addAction(clearPlaylistAction);
+    fileMenu->addAction(optionAction);
 //    fileMenu->addAction(exitAction);
 
     QMenu *aboutMenu = menuBar()->addMenu(tr("&Help"));
@@ -586,21 +654,27 @@ void MainWindow::setupMenus()
 void MainWindow::setupUi()
 {
     QToolBar *bar = new QToolBar;
+    bool flip = settings.value("uiflipped", false).toBool();
 
-    bar->setOrientation(Qt::Vertical);
+    if(!flip) bar->setOrientation(Qt::Vertical);
     bar->setStyleSheet("padding:7px");
-    //bar->addAction(volumeAction);
 
     seekSlider = new Phonon::SeekSlider(this);
     seekSlider->setMediaObject(mediaObject);
+    if (flip) seekSlider->setOrientation(Qt::Vertical);
 
     volumeSlider = new Phonon::VolumeSlider(this);
     volumeSlider->setAudioOutput(audioOutput);
-    volumeSlider->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
-    volumeSlider->setOrientation(Qt::Horizontal);
+    if (flip)
+    {
+        volumeSlider->setOrientation(Qt::Vertical);
+        volumeSlider->setMinimumHeight(150);
+    }
+    else
+    {
+        volumeSlider->setMinimumWidth(150);
+    }
     volumeSlider->setMuteVisible(false);
-//    volumeAddedAction = bar->addWidget(volumeSlider);
-//    volumeAddedAction->setVisible(false);
     bar->addAction(playAction);
     bar->addAction(pauseAction);
     bar->addAction(stopAction);
@@ -608,64 +682,86 @@ void MainWindow::setupUi()
     bar->addAction(shuffleAction);
     bar->addAction(nextAction);
     bar->addAction(previousAction);
+    bar->addAction(upAction);
+    bar->addAction(downAction);
+    bar->addAction(delAction);
+    bar->addAction(enqueueActionButton);
 
     contextMenu = new QMenu (this);
+    enqueueAction = contextMenu->addAction(tr ("Enqueue"));
     removeSelected = contextMenu->addAction(tr ("Remove selected"));
     removeAllButSelected = contextMenu->addAction(tr("Remove all but selected"));
     connect (removeSelected, SIGNAL (triggered()), this, SLOT (removeSelectedItem()));
     connect (removeAllButSelected, SIGNAL (triggered()), this, SLOT (removeAllButSelectedItem()));
+    connect (enqueueAction, SIGNAL (triggered()), this, SLOT (enqueueSelected()));
 
 
     timeLcd = new QLCDNumber;
 
-    QStringList headers;
-    headers << tr("Artist") << tr("Title") << tr("Album");
+    if (!headers.size ())
+    {
+        headers << "Artist" << "Title" << "Album";
+        settings.setValue("headers", headers);
+    }
 
-    musicTable = new QTableWidget(0, 3);
+    musicTable = new QTableWidget(0, headers.size ());
     musicTable->setHorizontalHeaderLabels(headers);
     musicTable->setSelectionMode(QAbstractItemView::SingleSelection);
     musicTable->setSelectionBehavior(QAbstractItemView::SelectRows);
+//    musicTable->setAlternatingRowColors(true);
+    musicTable->verticalHeader ()->hide();
     connect(musicTable, SIGNAL(cellDoubleClicked(int,int)),
         this, SLOT(tableClicked(int,int)));
     connect(musicTable, SIGNAL(cellClicked(int,int)),
         this, SLOT(cellClicked(int,int)));
-/*    for (int i = 0; i < 3; ++i)
-    {
-        if (!musicTable->horizontalHeaderItem(i))
-            continue;
-        musicTable->horizontalHeaderItem(i)->setBackgroundColor(QColor (128, 128, 255));;
-        musicTable->horizontalHeaderItem(i)->setForeground(QColor (255, 255, 255));
-    }*/
     for (int i = 0; i < musicTable->columnCount(); ++i)
     {
         QString lab = QString ("colWidth_%1").arg (i);
         int val = settings.value(lab, 0).toInt();
         if (val)
             musicTable->setColumnWidth(i, val);
-//        settings.setValue(lab, musicTable->columnWidth(i));
     }
 
 
-    QHBoxLayout *seekerLayout = new QHBoxLayout;
+    QLayout *seekerLayout;
+    QLayout *playbackLayout;
+    if (flip)
+    {
+        seekerLayout = new QVBoxLayout;
+        playbackLayout = new QHBoxLayout;
+        bar->addWidget(timeLcd);
+    }
+    else
+    {
+        seekerLayout = new QHBoxLayout;
+        playbackLayout = new QVBoxLayout;
+    }
     QToolBar* bar2 = new QToolBar;
     bar2->addAction(volumeAction);
     seekerLayout->addWidget(bar2);
     seekerLayout->addWidget(volumeSlider);
     seekerLayout->addWidget(seekSlider);
-    seekerLayout->addWidget(timeLcd);
-
-    QVBoxLayout *playbackLayout = new QVBoxLayout;
+    if (!flip)
+        seekerLayout->addWidget(timeLcd);
     volumeSlider->hide ();
     playbackLayout->addWidget(bar);
 
     QVBoxLayout *seekAndTableLayout = new QVBoxLayout;
 
     seekAndTableLayout->addWidget(musicTable);
-    seekAndTableLayout->addLayout(seekerLayout);
-
     QHBoxLayout *mainLayout = new QHBoxLayout;
     mainLayout->addLayout(seekAndTableLayout);
-    mainLayout->addLayout(playbackLayout);
+    if (flip)
+    {
+        seekAndTableLayout->addLayout(playbackLayout);
+        mainLayout->addLayout(seekerLayout);
+    }
+    else
+    {
+        seekAndTableLayout->addLayout(seekerLayout);
+        mainLayout->addLayout(playbackLayout);
+    }
+
 
     QWidget *widget = new QWidget;
     widget->setLayout(mainLayout);
@@ -678,6 +774,22 @@ void MainWindow::cellClicked(int /*row*/, int)
 {
 }
 
+void MainWindow::enqueueSelected()
+{
+    int sel = musicTable->currentRow();
+    qDebug () << "Enqueue on " << sel;
+    if (sel >= 0)
+    {
+        mediaObject->queue().clear();
+        mediaObject->enqueue(plman.at(sel));
+#ifdef Q_WS_MAEMO_5
+        QMaemo5InformationBox::information(this, plman.getItem(sel).title + tr (" enqueued as next song"),
+        QMaemo5InformationBox::DefaultTimeout);
+#endif
+
+    }
+}
+
 void MainWindow::contextMenuEvent (QContextMenuEvent*e)
 {
     if (!childAt (e->pos()))
@@ -714,7 +826,16 @@ void MainWindow::savePlaylist ()
     QString filename = QFileDialog::getSaveFileName(this, tr("Please select file name"), "", "Playlist Files (*.m3u *.pls)");
     if (filename.isEmpty())
         return;
-    plman.savePlaylist(filename);
+    if (!plman.savePlaylist(filename))
+    {
+#ifdef Q_WS_MAEMO_5
+        QMaemo5InformationBox::information(this, tr ("Error writing playlist file"),
+        QMaemo5InformationBox::DefaultTimeout);
+#else
+        QMessageBox::critical(this, "Write error", "Error writing playlist file", QMessageBox::Ok);
+#endif
+    }
+
 }
 
 void MainWindow::loadPlaylist ()
@@ -740,10 +861,10 @@ void MainWindow::playlistChanged(int from)
         musicTable->insertRow(currentRow);
         setRowFromItem (currentRow, plman.getItem(i));
     }
-    if (plman.indexOf(mediaObject->currentSource()) < 0)
+/*    if (plman.indexOf(mediaObject->currentSource()) < 0)
     {
         setItem (firstGood, false);
-    }
+    }*/
     setupShuffleList();
 }
 
@@ -753,21 +874,119 @@ void MainWindow::setRowFromItem (int row, const PlaylistItem& item)
         return;
     if (item.artist.isEmpty() && item.title.isEmpty())
     {
-        QTableWidgetItem *item1 = new QTableWidgetItem(item.uri);
-        item1->setFlags(item1->flags() ^ Qt::ItemIsEditable);
-        musicTable->setItem(row, 1, item1);
+        int col = headers.indexOf("Title");
+        if (col >= 0)
+        {
+            QTableWidgetItem *item1 = new QTableWidgetItem(item.uri);
+            item1->setFlags(item1->flags() ^ Qt::ItemIsEditable);
+            musicTable->setItem(row, col, item1);
+        }
     }
     else
     {
-        QTableWidgetItem *item1 = new QTableWidgetItem(item.artist);
-        item1->setFlags(item1->flags() ^ Qt::ItemIsEditable);
-        musicTable->setItem(row, 0, item1);
-        QTableWidgetItem *item2 = new QTableWidgetItem(item.title);
-        item2->setFlags(item2->flags() ^ Qt::ItemIsEditable);
-        musicTable->setItem(row, 1, item2);
-        QTableWidgetItem *item3 = new QTableWidgetItem(item.album);
-        item3->setFlags(item3->flags() ^ Qt::ItemIsEditable);
-        musicTable->setItem(row, 2, item3);
+        int col = headers.indexOf("Artist");
+        if (col >= 0)
+        {
+            QTableWidgetItem *item1 = new QTableWidgetItem(item.artist);
+            item1->setFlags(item1->flags() ^ Qt::ItemIsEditable);
+            musicTable->setItem(row, col, item1);
+        }
+        col = headers.indexOf("Title");
+        if (col >= 0)
+        {
+            if (!musicTable->item (row, col))
+            {
+                QTableWidgetItem *item2 = new QTableWidgetItem(item.title);
+                item2->setFlags(item2->flags() ^ Qt::ItemIsEditable);
+                musicTable->setItem(row, col, item2);
+            }
+            else
+            {
+                musicTable->item (row, col)->setText(item.title);
+            }
+        }
+        col = headers.indexOf("Album");
+        if (col >= 0)
+        {
+            QTableWidgetItem *item3 = new QTableWidgetItem(item.album);
+            item3->setFlags(item3->flags() ^ Qt::ItemIsEditable);
+
+            musicTable->setItem(row, col, item3);
+        }
+    }
+
+    int controlCol = headers.indexOf("Controls");
+    if (controlCol >= 0 && !musicTable->cellWidget(row, controlCol))
+    {
+        QLabel* label = new QLabel;
+        label->setText(QString::fromUtf8("<b><a style='text-decoration:none' href='up'>▲</a> <a style='text-decoration:none' href='down'>▼</a> <a style='text-decoration:none' href='del'>╳</a> <a style='text-decoration:none' href='enq'>E</a></b>"));
+        label->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
+        label->setProperty("row", row);
+        musicTable->setCellWidget(row, controlCol, label);
+        connect (label, SIGNAL (linkActivated (const QString&)),  this, SLOT (playlistControl (const QString&)));
+    }
+}
+
+void MainWindow::playlistControl (const QString& con)
+{
+    int i = sender ()->property("row").toInt();
+    if (con == "up")
+        buttonUp(i);
+    else if (con == "down")
+        buttonDown(i);
+    else if (con == "del")
+        buttonDel (i);
+    else if (con == "enq")
+    {
+        musicTable->selectRow(i);
+        enqueueSelected();
+    }
+    else
+        QMessageBox::information(this, tr ("Coming up..."), tr ("This feature is not implemented yet."));
+}
+
+
+void MainWindow::buttonUp(int i)
+{
+    if (i)
+    {
+        plman.moveItemUp(i);
+        setRowFromItem (i, plman.getItem(i));
+        setRowFromItem (i - 1, plman.getItem(i - 1));
+        int controlCol = headers.indexOf("Controls");
+        if (controlCol >= 0)
+        {
+            musicTable->cellWidget(i, controlCol)->setProperty("row", i);
+            musicTable->cellWidget(i - 1, controlCol)->setProperty("row", i - 1);
+        }
+        musicTable->selectRow(i - 1);
+    }
+}
+
+void MainWindow::buttonDown(int i)
+{
+    if (i < plman.size() - 1)
+    {
+        plman.moveItemDown(i);
+        setRowFromItem (i, plman.getItem(i));
+        setRowFromItem (i + 1, plman.getItem(i + 1));
+        int controlCol = headers.indexOf("Controls");
+        if (controlCol >= 0)
+        {
+            musicTable->cellWidget(i, controlCol)->setProperty("row", i);
+            musicTable->cellWidget(i + 1, controlCol)->setProperty("row", i + 1);
+        }
+        musicTable->selectRow(i + 1);
+    }
+}
+
+void MainWindow::buttonDel(int i)
+{
+    if (QMessageBox::question(this, "Confirm remove", "Are you sure you want to remove this item?", QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
+        return;
+    if (i < plman.size())
+    {
+        plman.removeItem(i);
     }
 }
 
@@ -784,3 +1003,56 @@ void MainWindow::itemUpdated(int index)
         setItem (index, false);
     }
 }
+
+void MainWindow::itemRemoved (int i)
+{
+    musicTable->removeRow(i);
+    int controlCol = headers.indexOf("Controls");
+    if (controlCol < 0)
+        return;
+    for (int j = i ? (i - 1) : 0; j < musicTable->rowCount(); ++j)
+    {
+        if (musicTable->cellWidget(j, controlCol))
+            musicTable->cellWidget(j, controlCol)->setProperty("row", j);
+    }
+}
+
+void MainWindow::upSelected()
+{
+    int sel = musicTable->currentRow();
+    if (sel > 0)
+        buttonUp(sel);
+}
+
+void MainWindow::downSelected()
+{
+    int sel = musicTable->currentRow();
+    if (sel >= 0)
+        buttonDown(sel);
+}
+
+
+
+void MainWindow::showOptions ()
+{
+    bool flip = settings.value("uiflipped", false).toBool();
+    OptionDialog* dlg = new OptionDialog (this, settings);
+    dlg->exec();
+    delete dlg;
+    if (flip != settings.value("uiflipped", false).toBool())
+    {
+        delete centralWidget();
+        setupUi ();
+    }
+    setOrientation ();
+    if (headers != settings.value("headers", QStringList ()).toStringList())
+    {
+        headers = settings.value("headers", QStringList ()).toStringList();
+        musicTable->setColumnCount(headers.size ());
+        musicTable->setHorizontalHeaderLabels(headers);
+        playlistChanged(0);
+    }
+    int curitem = plman.indexOf(mediaObject->currentSource());
+    if (curitem >= 0 && isPlaying)
+        highlightRow(curitem);
+}
